using PkmnLib.Dynamic.ScriptHandling;
using PkmnLib.Static;

namespace PkmnLib.Dynamic.Models.Choices;

/// <summary>
/// A choice to use an item
/// </summary>
public interface IItemChoice : ITurnChoice
{
    /// <summary>
    /// The item that is used.
    /// </summary>
    public IItem Item { get; }

    /// <summary>
    /// The side the move is targeted at.
    /// </summary>
    byte? TargetSide { get; }

    /// <summary>
    /// The position the move is targeted at.
    /// </summary>
    byte? TargetPosition { get; }
}

/// <inheritdoc cref="IItemChoice"/>
public class ItemChoice : TurnChoice, IItemChoice
{
    public ItemChoice(IPokemon user, IItem item, byte? targetSide, byte? targetPosition) : base(user)
    {
        Item = item;
        TargetSide = targetSide;
        TargetPosition = targetPosition;
    }

    public IItem Item { get; }

    /// <inheritdoc />
    public byte? TargetSide { get; }

    /// <inheritdoc />
    public byte? TargetPosition { get; }

    /// <inheritdoc />
    public override int ScriptCount => User.ScriptCount;

    /// <inheritdoc />
    public override void GetOwnScripts(List<IEnumerable<ScriptContainer>> scripts)
    {
    }

    /// <inheritdoc />
    public override void CollectScripts(List<IEnumerable<ScriptContainer>> scripts)
    {
        User.CollectScripts(scripts);
    }
}