using System.Collections.Generic;

namespace PkmnLib.Plugin.Gen7.Scripts.Moves;

public abstract class BaseChargeMove<TVolatile> : Script where TVolatile : Script
{
    public abstract TVolatile CreateVolatile(IPokemon user);

    public override void PreventMove(IExecutingMove move, ref bool prevent)
    {
        if (move.User.Volatile.Contains<TVolatile>())
            return;

        move.User.Volatile.Add(CreateVolatile(move.User));
        move.User.BattleData?.Battle.EventHook.Invoke(new DialogEvent("began_charging", new Dictionary<string, object>()
        {
            { "user", move.User },
        }));
        prevent = true;
    }

    /// <inheritdoc />
    public override void OnBeforeMove(IExecutingMove move)
    {
        move.User.Volatile.Remove(ScriptUtils.ResolveName<TVolatile>());
    }
}